part of masamune.flutter;

/// [IPath] Provider.
/// It can be used in the way riverpod is used.
///
/// You can also combine it with other providers.
///
/// You can use the notification feature of [IPath] as it is, and
/// When you receive a notification, the update will be notified to the widget etc.
///
/// ```dart
/// final counter = PathProvider((ref) => DataField("count"));
/// ```
class PathProvider<T extends IPath> extends AlwaysAliveProviderBase<T, T> {
  /// [IPath] Provider.
  /// It can be used in the way riverpod is used.
  ///
  /// You can also combine it with other providers.
  ///
  /// You can use the notification feature of [IPath] as it is, and
  /// When you receive a notification, the update will be notified to the widget etc.
  ///
  /// ```dart
  /// final counter = PathProvider((ref) => DataField("count"));
  /// ```
  ///
  /// [create]: Provider Definition.
  PathProvider(Create<T, ProviderReference> create) : super(create, null);

  /// An internal method that creates the state of a provider.
  @override
  _PathProviderState<T> createState() => _PathProviderState();
}

@sealed
class _PathProviderState<T extends IPath> = ProviderStateBase<T, T>
    with _PathProviderStateMixin<T>;

mixin _PathProviderStateMixin<T extends IPath> on ProviderStateBase<T, T> {
  @override
  void valueChanged({T previous}) {
    if (createdValue == previous) {
      return;
    }
    previous?.unwatch(_listener);
    exposedValue = createdValue;
    createdValue?.watch(_listener);
  }

  void _listener(T path) {
    exposedValue = createdValue;
  }

  @override
  void dispose() {
    createdValue?.unwatch(_listener);
    super.dispose();
  }
}
